package jalview.gui;

import java.awt.*;
import jalview.io.*;
import jalview.analysis.NJTree;
import jalview.datamodel.*;
import jalview.schemes.*;
import java.util.*;

public class AlignViewport
{
  int startRes;
  int endRes;

  int startSeq;
  int endSeq;

  boolean showScores=false;
  boolean showText=true;
  boolean showColourText=false;
  boolean showBoxes=true;
  boolean wrapAlignment=false;
  boolean renderGaps = true;
  boolean showSequenceFeatures = false;

  boolean colourAppliesToAllGroups = true;
  ColourSchemeI globalColourScheme = null;
  boolean conservationColourSelected = false;
  boolean abovePIDThreshold = false;

  SequenceGroup selectionGroup = new SequenceGroup();

  RendererI renderer = new SequenceRenderer(this);

  int             charHeight;
  int             charWidth;
  int             chunkWidth;
  int             chunkHeight;

  Font            font = new Font("SansSerif",Font.PLAIN,10);
  AlignmentI      alignment;

  ColumnSelection colSel = new ColumnSelection();

  String visibleConsensus;

  int threshold;
  int increment;

  NJTree currentTree = null;


  public AlignViewport(AlignmentI da,
                       boolean showScores,
                       boolean showText,
                       boolean showBoxes,
                       boolean wrapAlignment) {
    this(0,da.getWidth()-1,0,da.getHeight()-1,showScores,
         showText,
         showBoxes,
         wrapAlignment);

    setAlignment(da);
  }

  public AlignViewport(int startRes, int endRes,
                       int startSeq, int endSeq,
                       boolean showScores,
                       boolean showText,
                       boolean showBoxes,
                       boolean wrapAlignment) {

    this.startRes = startRes;
    this.endRes   = endRes;
    this.startSeq = startSeq;
    this.endSeq   = endSeq;

    this.showScores    = showScores;
    this.showText      = showText;
    this.showBoxes     = showBoxes;
    this.wrapAlignment = wrapAlignment;

   // og  = new AlignmentOutputGenerator(this);

    setFont( font );
 }

 public void showSequenceFeatures(boolean b)
 {
   showSequenceFeatures = b;
 }

  public String getVisibleConsensus()
  {
    return visibleConsensus;
  }

  Vector consensus = new Vector();
  public Vector getConsensus(boolean recalculate)
  {
    if(recalculate || consensus.size()<1)
    {
      consensus = alignment.getAAFrequency();
      StringBuffer sb = new StringBuffer();
      Hashtable hash = null;
      for (int i = 0; i < consensus.size(); i++)
      {
        hash = (Hashtable) consensus.elementAt(i);
        sb.append(hash.get("maxResidue").toString().charAt(0));
      }
      visibleConsensus = sb.toString();
    }


    return consensus;
  }

  public SequenceGroup getSelectionGroup()
  {
    return selectionGroup;
  }

  public void setSelectionGroup(SequenceGroup sg)
  {
    selectionGroup = sg;
  }


 public boolean getConservationSelected()
 {
   return conservationColourSelected;
 }

 public void setConservationSelected(boolean b)
 {
   conservationColourSelected = b;
 }

 public boolean getAbovePIDThreshold()
 {
   return abovePIDThreshold;
 }

 public void setAbovePIDThreshold(boolean b)
 {
   abovePIDThreshold = b;
 }

  public int getStartRes() {
    return startRes;
  }

  public int getEndRes() {
    return endRes;
  }

  public int getStartSeq() {
    return startSeq;
  }

  public void setGlobalColourScheme(ColourSchemeI cs)
  {
     globalColourScheme = cs;
  }

  public ColourSchemeI getGlobalColourScheme()
  {
    return globalColourScheme;
  }


  public void setStartRes(int res) {
    this.startRes = res;
  }
  public void setStartSeq(int seq) {
    this.startSeq = seq;
  }
  public void setEndRes(int res) {
    if (res > alignment.getWidth()-1) {
      System.out.println(" Corrected res from " + res + " to maximum " + (alignment.getWidth()-1));
       res = alignment.getWidth() -1;
    }
    if (res < 0) {
      res = 0;
    }
    this.endRes = res;
  }
  public void setEndSeq(int seq) {
    if (seq > alignment.getHeight()) {
      seq = alignment.getHeight();
    }
    if (seq < 0) {
      seq = 0;
    }
    this.endSeq = seq;
  }
  public int getEndSeq() {
    return endSeq;
  }

  public void setFont(Font f) {
    font = f;
    javax.swing.JFrame temp = new javax.swing.JFrame();
    temp.addNotify();
    java.awt.FontMetrics fm = temp.getGraphics().getFontMetrics(font);
    setCharHeight(fm.getHeight());
    setCharWidth(fm.charWidth('M'));
  }

  public Font getFont() {
    return font;
  }
  public void setCharWidth(int w) {
    this.charWidth = w;
  }
  public int getCharWidth() {
    return charWidth;
  }
  public void setCharHeight(int h) {
    this.charHeight = h;
  }
  public int getCharHeight() {
    return charHeight;
  }
  public void setChunkWidth(int w) {
    this.chunkWidth = w;
  }
  public int getChunkWidth() {
    return chunkWidth;
  }
  public void setChunkHeight(int h) {
    this.chunkHeight = h;
  }
  public int getChunkHeight() {
    return chunkHeight;
  }
  public AlignmentI getAlignment() {
    return alignment;
  }
  public void setAlignment(AlignmentI align) {
    this.alignment = align;
  }
  public void setShowScores(boolean state) {
    showScores = state;
  }
  public void setWrapAlignment(boolean state) {
    wrapAlignment = state;
  }
  public void setShowText(boolean state) {
    showText = state;
  }

  public void setRenderGaps(boolean state){
    renderGaps = state;
    if(renderer instanceof SequenceRenderer)
    {
      SequenceRenderer sr = (SequenceRenderer)renderer;
      sr.renderGaps(state);
    }
  }


  public boolean getColourText()
  {
    return showColourText;
  }

  public void setColourText(boolean state)
  {
    showColourText = state;
  }

  public void setShowBoxes(boolean state) {
    showBoxes = state;
  }
  public boolean getShowScores() {
    return showScores;
  }
  public boolean getWrapAlignment() {
      return wrapAlignment;
  }
  public boolean getShowText() {
    return showText;
  }
  public boolean getShowBoxes() {
    return showBoxes;
  }

  public char getGapCharacter() {
    return getAlignment().getGapCharacter();
  }
  public void setGapCharacter(char gap) {
    if (getAlignment() != null) {
      getAlignment().setGapCharacter(gap);
    }
  }
  public void setThreshold(int thresh) {
    threshold = thresh;
  }
  public int getThreshold() {
    return threshold;
  }
  public void setIncrement(int inc) {
    increment = inc;
  }
  public int getIncrement() {
    return increment;
  }
  public int getIndex(int y) {
    int y1     = 0;
    int starty = getStartSeq();
    int endy   = getEndSeq();

    for (int i = starty; i <= endy; i++) {
      if (i < alignment.getHeight() && alignment.getSequenceAt(i) != null) {
        int y2 = y1 + getCharHeight();

        if (y>=y1 && y <=y2) {
          return i;
        }
        y1  = y2;
      } else {
        return -1;
      }
    }
    return -1;
  }

  public ColumnSelection getColumnSelection() {
    return colSel;
  }

  public void resetSeqLimits(int height) {
    setStartSeq(0);
    setEndSeq(height/getCharHeight());
  }
  public void setCurrentTree(NJTree tree) {
      currentTree = tree;
  }
  public NJTree getCurrentTree() {
    return currentTree;
  }

    public void setRenderer(RendererI rend) {
	this.renderer = rend;
    }

    public RendererI getRenderer() {
	return renderer;
    }

  public void setColourAppliesToAllGroups(boolean b)
  {   colourAppliesToAllGroups = b; }

  public boolean getColourAppliesToAllGroups()
  {return colourAppliesToAllGroups; }


}
